// kolmogorov-smirnov
//
// Takes a file of samples and does kolmogorov-smirnov test
// that it is fit by some distribution

#include "math.h"
#include "stdio.h"
#include "stdlib.h"

int rvcmp(const void *rv1,const void *rv2)
{
    if ((*(double *)rv1) < (*(double *)rv2)) return(-1);
    if ((*(double *)rv1) > (*(double *)rv2)) return(1);
    return(0);
}

double ks(double t)
// compute P(root(n)*D <= t) 
{
double total=0.;
for (long k=1;k<=100;k++) 
    if (k%2==0) total-=exp(-2*k*k*t*t);
    else total+=exp(-2*k*k*t*t);
 return(1.-2*total);
}

int main()
{
#define MAXN 100000

double rv,*samples,z,fz,mean,variance;
FILE *fptr;
long i;
char fname[101];

printf("name of file with sample\n");
scanf("%s",fname);
mean=0.;
double dt=0.002;
double kappa=2.66666;
variance=dt*kappa;
 
fptr=fopen(fname,"r"); 
if (fptr==NULL) {printf("file does not exist \n"); exit(0);}
long nsample=0;
while (fscanf(fptr,"%le",&rv)!=-1) nsample++;
fclose(fptr);

samples=new double[nsample];
fptr=fopen(fname,"r"); 
for (i=0;i<nsample;i++) fscanf(fptr,"%le",samples+i);
fclose(fptr);

printf("nsample=%ld\n",nsample); 

qsort(samples,nsample,sizeof(double),rvcmp);

double max=0.,dif;
for (i=0;i<nsample;i++) {
    z=(samples[i]-mean)/sqrt(variance); 
    if (z<0.) fz=0.5*(1.-erf(-z/sqrt(2.)));
    else fz=0.5*(1.+erf(z/sqrt(2.)));
    dif=fabs(fz-(double(i)+0.5)/double(nsample));
    if (dif>max) max=dif;
}
double ksd=max+0.5/double(nsample);

printf("number of samples is %ld \n",nsample); 
printf("mean=%lf variance=%lf\n",mean,variance);
printf("D= %lf \n",ksd);
printf("root(n)*D= %lf\n",sqrt(double(nsample))*ksd);
printf("P(D>above)=%lf=%le \n",1-ks(sqrt(double(nsample))*ksd),
    1-ks(sqrt(double(nsample))*ksd));
printf("P(D>1.36)=%lf\n",1-ks(1.36));
}

