// file name : src/rpoint.h
//
// class for points with real coords (as opposed to integer)

#ifndef __rpoint_hh
#define __rpoint_hh

class point;
class excluded_class;

class rpoint {
private:
    // these are actual x,y coordinates 
    #ifdef TWO_DIMENSIONS
        double x,y;
    #endif
    #ifdef THREE_DIMENSIONS
        double x,y,z;
    #endif
    #ifdef FOUR_DIMENSIONS
        double x,y,z,w;
    #endif
public:
    void print(FILE *fptr);
    void scanf(FILE *fptr);
    void write(FILE *fptr);
    void read(FILE *fptr);

    inline rpoint& operator+=(rpoint p) 
    {
        x+=p.x;
        y+=p.y;
        #ifdef THREE_DIMENSIONS
            z+=p.z;
        #endif
        #ifdef FOUR_DIMENSIONS
            z+=p.z;
            w+=p.w;
        #endif
        return *this;
    }

    inline rpoint& operator-=(rpoint p) 
    {
        x-=p.x;
        y-=p.y;
        #ifdef THREE_DIMENSIONS
            z-=p.z;
        #endif
        #ifdef FOUR_DIMENSIONS
            z-=p.z;
            w-=p.w;
        #endif
       return *this;
    }

    rpoint& operator*=(double c);
    rpoint& operator/=(double c);
    void zero(); // sets coords to zero 
    rpoint& operator=(point p); 
    void rotate(double theta); 
    double distance(); 
    double distance_projection(double a, double b);
    double p_angle();
    double coord_x();
    double coord_y();
    #ifdef TWO_DIMENSIONS
        void assign(double xx,double yy);
    #endif
    #ifdef THREE_DIMENSIONS
        void assign(double xx,double yy,double zz);
        double coord_z();
    #endif
    #ifdef FOUR_DIMENSIONS
        void assign(double xx,double yy,double zz,double ww);
        double coord_z();
        double coord_w();
    #endif
};  // end class rpoint

inline rpoint operator+(rpoint p,rpoint q) 
{
    rpoint r=p; 
    return r+=q;
} 

inline rpoint operator-(rpoint p,rpoint q) 
{
    rpoint r=p; 
    return r-=q;
} 

rpoint operator*(double c,rpoint p);
rpoint operator*(rpoint p,double c);
rpoint operator/(rpoint p,double c);
rpoint operator*(rpoint p,rpoint q);
inline double inner_prod(rpoint* p,rpoint* q);

inline double inner_prod(rpoint* p,rpoint* q) 
// computes inner product of the two rpoints
{
    #ifdef TWO_DIMENSIONS
        return((*p).coord_x()*(*q).coord_x()+(*p).coord_y()*(*q).coord_y());
    #endif
    #ifdef THREE_DIMENSIONS
        return((*p).coord_x()*(*q).coord_x()+(*p).coord_y()*(*q).coord_y()
                             +(*p).coord_z()*(*q).coord_z());
    #endif
    #ifdef FOUR_DIMENSIONS
        return((*p).coord_x()*(*q).coord_x()+(*p).coord_y()*(*q).coord_y()
           +(*p).coord_z()*(*q).coord_z()+(*p).coord_w()*(*q).coord_w());
    #endif
} 


#endif

///////////////////////////////////////////////////////////////
//  checkk checkk checkk checkk checkk checkk checkk checkk  //
///////////////////////////////////////////////////////////////
