// file : src/rwalk.c 
//  member functions for class rwalk that are not part of public release

////////////////////////////////////////////////////////////////
//             member functions for class rwalk               //
////////////////////////////////////////////////////////////////

// system includes
#include <cstdio> 
using namespace std; 
#include "math.h"
#include <iostream>
using namespace std; 
#include <cstdlib>
#include "sys/time.h"
#include "time.h"
#include "limits.h"
#include <complex>
using namespace std;

#define I complex<double>(0.,1.);

// local includes
#include "random_number_generator.h"
#include "local_constants.h"
#include "lib.h"
#include "point.h"
#include "rpoint.h"
#include "excluded_class.h"
#include "excluded_class.h"
#include "rwalk.h"
#include "walk.h"

int rwalk::allocate(long n)
{
    steps = new rpoint[n+1];
    if (steps==0) { //allocation failed
        cout << "allocate error in rwalk::allocate \n";
        return(0);
    }
    nsteps=n; 
    return(1);
} 

void rwalk::deallocate() 
{
    delete [] steps;
} 

void rwalk::print(FILE *fptr) 
// prints the number of steps and coordinates of the rwalk
{
    fprintf(fptr,"%ld\n",nsteps);
    for (long i=0;i<=nsteps;i++) steps[i].print(fptr);
} 

void rwalk::plot(FILE *fptr)
// prints the coordinates of the rwalk for plotting
{ 
for (long i=0;i<=nsteps;i++)  
    fprintf(fptr,"%lf %lf\n",steps[i].coord_x(),steps[i].coord_y());
} 

rpoint rwalk::step_rval(long i) 
{
    return(steps[i]);
}

void rwalk::assign_step(long i,rpoint pt)
{
    steps[i]=pt;
}

void rwalk::interpolate(long ninterp)
{
    for (long i=nsteps;i>0;i--) {
        for (long j=0;j<ninterp;j++) {
            double alpha=double(j)/double(ninterp);
            steps[i*ninterp-j]=(1-alpha)*steps[i]+alpha*steps[i-1];
        }   
    }
    nsteps*=ninterp;
} 

void rwalk::time_reverse()
// w[i]=w[nsteps-i]
{
    rpoint temp;
    for (long i=0;2*i<nsteps;i++) {
        temp=steps[i];
        steps[i]=steps[nsteps-i];
        steps[nsteps-i]=temp;
    }
}


#ifdef TWO_DIMENSIONS
void rwalk::rotate(double theta) 
{
    for (long i=0;i<=nsteps;i++) steps[i].rotate(theta);
} 
#endif

void rwalk::rescale(double c) 
{
    for (long i=0;i<=nsteps;i++) steps[i]*=c;
}

rwalk& rwalk::operator=(walk& w)
{
    if (w.get_npivot()!=0) {
        printf("a ERROR in rwalk::=, npivot!=0 \n");
        exit(1);
    } 
    nsteps=w.get_nsteps();
    for (long i=0;i<=nsteps;i++) steps[i]=w.step_rval(i);
    return(*this);
} 

///////////////////////////////////////////////////////////////
//  checkk checkk checkk checkk checkk checkk checkk checkk  //
///////////////////////////////////////////////////////////////

