// file: src/simul_saw_bridge.c
// Runs the pivot algorithm to generate SAW bridges.
//
// checks for a bridge and if there is one, dumps the walk up to
// the bridge or just the bridge point to file.
// In first version of this program, bridge was just a vertical line
// that the walk intersected once.
// This has been generalized. Now we specify a curve and
// a bridge occurs when the walk intersects this curve exactly once.
//
// bridge_type specifies the type of curve:
//  =1 for horizontal line
//  =2 for semi-circle
//  =3 for rectangle
//  = -2 for full plane walk with circle are cut curve
//
//
// bridge_parms[] contains parameters needed to specify the curve
//

// system includes
#include <cstdio>
using namespace std;
#include "math.h"
#include <iostream>
using namespace std;
#include <cstdlib>
#include "sys/time.h"
#include "time.h"
#include "limits.h"
#include <complex>
using namespace std;

#define I complex<double>(0.,1.);

// local includes
#include "random_number_generator.h"
#include "local_constants.h"
#include "lib.h"
#include "rpoint.h"
#include "point.h"
#include "excluded_class.h"
#include "rwalk.h"
#include "walk.h"

extern long group_inverse[NUM_SYM+1];

int main()
{
walk w;
excluded_class excluded;
long ntries,naccept,iseed,num_bridge,max_num_bridge;
long pivot_type,pivot_loc,iiter,inner;
char init_fname[101],final_fname[101],data_fname[101];
// sign of accept flag indicates walk was accepted (>0) or rejected (<0)
// |accept_flag| is the number of tests done in the call to pivot routine
long accept_flag=0;
double accept_ratio=0.;
rpoint endpt;
double scale,nu;
FILE *fptr;
long nsteps,region,nsimplify,niter,nsample,interaction,direction,flag_walk;
long bridge_nsteps;
double beta,p,etime;
long bridge_type;
double bridge_parms[10];

setup_symmetry();

printf("Enter name of file containing initial walk, \n");
printf(
   " or 0 to have program generate a straight line for initial walk (0)\n");
scanf("%s",init_fname);
printf("Enter name of file in which to store final walk (junkwalk) \n");
scanf("%s",final_fname);
printf("Enter name of file in which to store data (junkdata) \n");
scanf("%s",data_fname);

printf("seed for random number generator (0) \n");
scanf("%ld",&iseed);
printf("number of steps in the walk (100000) \n");
scanf("%ld",&nsteps);

nu=0.75;
scale=double(nsteps);
scale=pow(scale,nu);

region=1;

printf("1 for strictly or 2 for weakly or 3 for FWSAW (1)\n");
scanf("%ld",&interaction);
switch (interaction) {
    case 0:
    break;
    case 1:
    break;
    case 2:
        printf("beta\n");
        scanf("%lf",&beta);
    break;
    case 3:
        printf("beta\n");
        scanf("%lf",&beta);
        printf("p\n");
        scanf("%lf",&p);
    break;



    default:
        printf("interaction case not implemented in simul.c\n");
        exit(1);
    break;
}

printf("nsimplify (0)\n");
scanf("%ld",&nsimplify);
printf("max_num_bridge  (100)\n");
scanf("%ld",&max_num_bridge);
printf("nsample (100000) \n");
scanf("%ld",&nsample);
// bridge type parms
printf(
  "bridge type, 1=horizontal line, 2=circle, -2=circle for full plane (1)\n 3=rectangle\n 4=wedge");
scanf("%ld",&bridge_type);
switch(bridge_type) {
 case 1:
  printf("height  (0.2)\n");
  scanf("%lf",bridge_parms+0);
  bridge_parms[0]*=scale;
 break;
 case 2:
  printf("radius\n");
  scanf("%lf",bridge_parms+0);
  bridge_parms[0]*=scale;
 break;
 case -2:
  printf("radius\n");
  scanf("%lf",bridge_parms+0);
  bridge_parms[0]*=scale;
  region=0;
 break;
 case 3:
  printf("for square, x and y should be equal\n");
  printf("x (half of width) \n");
  scanf("%lf",bridge_parms+0);
  bridge_parms[0]*=scale;
  printf("y (height) \n");
  scanf("%lf",bridge_parms+1);
  bridge_parms[1]*=scale;
 break;

 case 4 :
  printf("for wedge pick slope and intercept\n");
    printf("slope \n");
    scanf("%lf",bridge_parms+0);

    printf("x-intercept \n");
    scanf("%lf",bridge_parms+1);
    bridge_parms[1]*=scale;

 break;

 default:
  printf("bridge type not implemented \n");
  exit(1L);
 break;
}
excluded.region=region;


printf("just endpoint or walk? 0=endpt, 1=walk  (0)\n");
scanf("%ld",&flag_walk);

// nsimplify=0 => compute nsimplify
if (nsimplify==0) nsimplify=long(sqrt(double(nsteps/40)));

// seed random number generator
initialize_random_number_generator(iseed);

/////////////////////////////////////////////////////////////////
//                    allocate memory                          //
/////////////////////////////////////////////////////////////////

printf("nsteps=%ld\n",nsteps);
w.allocate(nsteps);

////////////////////////////////////////////////////////////
//  initialize walk to a line or  read walk from file     //
////////////////////////////////////////////////////////////

// if filename for initial walk starts with a 0 we generate a line
// for the initial walk. direction is its direction:
// 1=horizontal, 2=45 degs, 3=vertical, 4=60 degs
direction=3;
if (init_fname[0]=='0') w.line_initialize(direction);
else {
    fptr=fopen(init_fname,"r");
    if (fptr==NULL) {
        printf("FAILURE to open walk file %s, exiting \n",init_fname);
        exit(0);
    }
    w.scan(fptr);
    fclose(fptr);
}

// checks that initial walk is ok
w.test(&(excluded));
printf("Initial walk has turn fraction %f \n",w.turn_frac());

// initialize running totals to zero
ntries=0;
naccept=0;
num_bridge=0;
etime=-mytime();


// record parameters used
fptr=fopen(data_fname,"w");

fprintf(fptr,"nsteps   %10ld\n",nsteps);
fprintf(fptr,"nsample  %10ld\n",nsample);
fprintf(fptr,"bridge_type %10ld\n",bridge_type);
switch(bridge_type) {
 case 1:
   fprintf(fptr,"height   %10.4f\n",bridge_parms[0]);
 break;
 case 2:
 case -2:
   fprintf(fptr,"radius   %10.4f\n",bridge_parms[0]);
 break;
}
fclose(fptr);

for (iiter=1;num_bridge<max_num_bridge;iiter++) {
    for (inner=1;inner<=nsample;inner++) {
        ntries++;

        // a pivot location at nsteps will do nothing, but a pivot at 0 will
        pivot_loc=int(floor(nsteps*RNG()));

        ///////////////// MODEL DEPENDENT ////////////////////////////
        #if ALGORITHM_NUMBER==1
        // Manhattan lattice. Lattice operation is
        //   reflect in +45 deg line (isym=1) if pivot point (x,y) has x+y even
        //   reflect in -45 deg line (isym=2) if pivot point (x,y) has x+y odd
        // so choice of lattice sym op is not random
            if (w.even(pivot_loc)==1) pivot_type=1;
            else pivot_type=2;
        #endif

        #if ALGORITHM_NUMBER==2
        // sq lattice with only reflections in 45 deg and
        // -45 degree lines (this is supposedly ergodic)
            pivot_type=int(floor(2*RNG()+1));
        #endif

        #if ALGORITHM_NUMBER==7
        // regular sq lattice with all 7 nontrivial lattice symmetries
            pivot_type=int(floor(7*RNG()+1));
        #endif

        #if ALGORITHM_NUMBER==5
        // hexagonal lattice with 5 nontrivial lattice symmetries
            pivot_type=int(floor(5*RNG()+1));
        #endif

        #if ALGORITHM_NUMBER==11
        // triangular lattice with all 11 nontrivial lattice symmetries
            pivot_type=int(floor(11*RNG()+1));
        #endif

        #if ALGORITHM_NUMBER==47
        // cubic lattice with all 47 nontrivial lattice symmetries
            pivot_type=int(floor(47*RNG()+1));
        #endif

        #if ALGORITHM_NUMBER==383
        // 4d hypercubic lattice with all 383 nontrivial lattice symmetries
            pivot_type=int(floor(383*RNG()+1));
        #endif

        switch (interaction) {
            case 0:
                accept_flag=w.pivot_ordinary_rw(pivot_loc,pivot_type,excluded);
            break;
            case 1:
               accept_flag=w.pivot_strictly_saw(pivot_loc,pivot_type,excluded);
            break;
            case 2: // note we send p=0 to the routine here
                accept_flag=
                    w.pivot_weakly_saw(pivot_loc,pivot_type,beta,0.,excluded);
            break;
            case 3:
                accept_flag=
                    w.pivot_weakly_saw(pivot_loc,pivot_type,beta,p,excluded);
            break;
        }
        if (w.get_npivot()==nsimplify)  w.simplify();
        if (accept_flag> 0) naccept++;

    } // end loop on inner

    w.increment_niter();
    w.simplify();

    bridge_nsteps=w.if_bridge(bridge_type,bridge_parms);
    printf(">>>>>>>>> %ld %ld %ld %lf secs <<<<<<< \n",
       iiter,bridge_nsteps,num_bridge,etime+mytime());
    if (bridge_nsteps>0) {
        num_bridge++;
        printf("bridge_nsteps=%ld \n",bridge_nsteps);

        endpt=w.step_rval(bridge_nsteps);
        endpt/=scale;

        accept_ratio=100*double(naccept)/double(ntries);
        printf("%ldK iterations, turn=%f, accept ratio=%f, %lf secs\n",
            (inner/1000+(iiter-1)*(nsample/1000)),
            w.turn_frac(),accept_ratio,etime+mytime());
        endpt.print(stdout);

        ntries=0;
        naccept=0;

        // record the bridge
        w.set_nsteps(bridge_nsteps);
        fptr=fopen(data_fname,"a");
        fprintf(fptr,"\n");
        fprintf(fptr,"bridge %6ld %10ld %8ld\n",
            num_bridge,bridge_nsteps,iiter);
        if (flag_walk) w.iplot(fptr);
        else fprintf(fptr,"%14.10lf %14.10lf\n",
            endpt.coord_x(),endpt.coord_y());
        fclose(fptr);
        w.set_nsteps(nsteps);

    }

} // end loop on iiter

// record the walk itself
fptr=fopen(final_fname,"w");
w.print(fptr);
fclose(fptr);

} // end main

///////////////////////////////////////////////////////////////
//  checkk checkk checkk checkk checkk checkk checkk checkk  //
///////////////////////////////////////////////////////////////
