// file name : src/walk.h
//


#ifndef __walk_hh
#define __walk_hh

void setup_symmetry();

class walk {
// stores a single walk
private: 
protected:
    point* steps;
    long nsteps; // number of steps in the walk  
    long niter; // number of attempted pivots the walk has been through 
      // since it was a line. In multiples of inner_loop
      // So if inner_loop=1,000,000, this is number in millions
    long npivot; // number of accepted pivots not yet applied to walk
    long* ptime; // array of pivot times 
    long* igroup; // array of group elements  
    point* shift; // array of shifts
public:
    long num_unzip; // used to record number of steps that were unzipped
    int allocate(long n);
    void deallocate(); 
    void initialize(); 
    long get_nsteps();
    void set_nsteps(long n);
    long get_niter();
    void increment_niter();
    long get_npivot();
    void set_npivot(long n);
    point step_rval(long i);
    void assign_point(long i,point p);
    void print(FILE *fptr);
    void scan(FILE *fptr);
    void plot(FILE *fptr);
    void iplot(FILE *fptr);
    void bi_plot(FILE *fptr1,FILE *fptr2,long nseg); 
    void plot_projected(FILE *fptr,double rrho);
    long check_excluded(long pivot_loc,long isym,excluded_class excluded,
        long direction);
    long test_pivot_strictly_saw(
        long pivot_loc,long isym,excluded_class excluded);
    long test_pivot_bond_saw(long pivot_loc,long isym,excluded_class excluded);
    long pivot_ordinary_rw(long pivot_loc,long isym,excluded_class excluded);
    long pivot_strictly_saw(long pivot_loc,long isym,excluded_class excluded);
    long pivot_bond_saw(long pivot_loc,long isym,excluded_class excluded);

    long pivot_strictly_two_saw(long pivot_loc,long isym,
        excluded_class excluded);
  
    long pivot_weakly_saw(long pivot_loc,long isym,double beta,double p,
        excluded_class excluded);
    walk& operator=(walk& w);
    void line_initialize(int direction);
    void two_line_initialize(int direction);
    void add_pivot(long pivot_loc,long isym,point trans);
    void simplify(); //carry out the pivots, so npivot -> 0
    double turn_frac(); // fraction of sites with a turn in the walk
    void test(excluded_class* excluded); // checks walk is n.n.,misses excluded
    int even(long pivot_loc); 
    void rw_outer_loop(long ns,long *hloc,point *htab_bond);
    double max_radius();
    double area();
    double diameter();

    long if_bridge(long bridge_type,double *bridge_parms);
}; // end class walk 

inline long find_segment(long itime,long npivot,long* ptime) 
// Finds the segment number for a pivot time. 
// segment number iseg corresponds to times [ptime[iseg],ptime[iseg+1])
// Note that the left endpt, ptime[iseg], is included in segment iseg.
{
long iseg,isegl,isegu;

if (itime>=ptime[npivot]) return(npivot);

isegl=0; isegu=npivot; iseg=0;

while (isegu>isegl+1)
 {
  iseg=(isegl+isegu)/2;
  if (itime<ptime[iseg]) isegu=iseg;
  else isegl=iseg;
 }
return(isegl);
} // find_segment()

#endif

///////////////////////////////////////////////////////////////
//  checkk checkk checkk checkk checkk checkk checkk checkk  //
///////////////////////////////////////////////////////////////
